//------------------------------------------------------------------
/*
  cfb adaptive T3 velocity histo mtf.mq4

 To "give it room" widen the ShortLimit to LongLimit range
(defaults are set to resemble the original T3 velocity) One more 
possibility is "misused" in this indicator internally : T3 period 
does not have to be integer so this indicator almost never calculates 
some integer length T3 velocity (the same goes for ShortLimit and 
longLimit - they do not have to be integer values)

The original refers to the way how T3 is calculated.
If you set it to true then original Tim Tillson (the inventor of T3) 
calculation is used. If it is set to false, then the 
Bob Fulks/Alex Matulich variation (faster calculation) is used

http://www.forex-tsd.com/advanced-elite/22691-composite-fractal-behavior-its-aplications-16.html#post434048
*/
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"


#property indicator_separate_window
#property indicator_buffers    5
#property indicator_color1     LimeGreen
#property indicator_color2     LimeGreen
#property indicator_color3     PaleVioletRed
#property indicator_color4     PaleVioletRed
#property indicator_color5     DeepSkyBlue
#property indicator_width1     2
#property indicator_width3     2
#property indicator_width5     2
#property indicator_level1     0
#property indicator_levelcolor White

//
//
//
//
//

extern string TimeFrame     = "Current time frame";
extern double ShortLimit    = 3;
extern double LongLimit     = 15;
extern double Hot           = 1.0;
extern int    Price         = PRICE_CLOSE;
extern bool   Original      = false;
extern int    CfbDepth      = 6;
extern int    CfbSmooth     = 8;
extern int    CfbNormLength = 50;
extern bool   Interpolate   = true;

//
//
//
//
//

double vel[];
double cfb[];
double velHuu[];
double velHud[];
double velHdd[];
double velHdu[];
double slope[];
double trend[];

//
//
//
//
//

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//------------------------------------------------------------------
//   
//------------------------------------------------------------------
//
//

int init()
{
   IndicatorBuffers(8);
   SetIndexBuffer(0,velHuu); SetIndexStyle(0, DRAW_HISTOGRAM);
   SetIndexBuffer(1,velHud); SetIndexStyle(1, DRAW_HISTOGRAM);
   SetIndexBuffer(2,velHdd); SetIndexStyle(2, DRAW_HISTOGRAM);
   SetIndexBuffer(3,velHdu); SetIndexStyle(3, DRAW_HISTOGRAM);
   SetIndexBuffer(4,vel);
   SetIndexBuffer(5,cfb);
   SetIndexBuffer(6,slope);
   SetIndexBuffer(7,trend);

       //
       //
       //
       //
       //
       
       indicatorFileName = WindowExpertName();
       returnBars        = (TimeFrame == "returnBars");     if (returnBars)     return(0);
       calculateValue    = (TimeFrame == "calculateValue"); if (calculateValue) return(0);
       timeFrame         = stringToTimeFrame(TimeFrame); 
       
       //
       //
       //
       //
       //
   
   IndicatorDigits(5);
   IndicatorShortName(timeFrameToString(timeFrame)+"  cfb adaptive T3 velocity histo("+DoubleToStr(ShortLimit,2)+","+DoubleToStr(LongLimit,2)+","+DoubleToStr(Hot,2)+")");
   SetIndexLabel(0,"");
   SetIndexLabel(1,"");
   SetIndexLabel(2,"");
   SetIndexLabel(3,"");
   SetIndexLabel(4,"");
   SetIndexLabel(5,"");
   SetIndexLabel(6,"");
   SetIndexLabel(7,"");
   return(0);
}

//------------------------------------------------------------------
//  
//------------------------------------------------------------------
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { velHuu[0] = MathMin(limit+1,Bars-1); return(0); }

   //
   //
   //
   //
   //
   
   if (calculateValue || timeFrame==Period())
   {
     for(int i=limit; i>=0; i--)
     {
        double price  = iMA(NULL,0,1,0,MODE_SMA,Price,i);
        cfb[i] = iCfb(price,CfbDepth,CfbSmooth,i);
    
        //
        //
        //
        //
        //
         
        double max = cfb[ArrayMaximum(cfb,CfbNormLength,i)];
        double min = cfb[ArrayMinimum(cfb,CfbNormLength,i)];
        double denominator = max-min;
           if (denominator> 0)
                 double ratio = (cfb[i]-min)/denominator;
           else         ratio = 0.5;  
           double Length = MathMax(ShortLimit+ratio*(LongLimit-ShortLimit),3); 
 
           //
           //
           //
           //
           //
         
           vel[i] = iT3(price,Length,Hot,Original,i,0)-iT3(price,Length,Hot/2.0,Original,i,1);
           velHuu[i] = EMPTY_VALUE;
           velHud[i] = EMPTY_VALUE;
           velHdd[i] = EMPTY_VALUE;
           velHdu[i] = EMPTY_VALUE;
           trend[i]  = trend[i+1];
           slope[i]  = slope[i+1];
           if (vel[i]>0)        trend[i] =  1;
           if (vel[i]<0)        trend[i] = -1;
           if (vel[i]>vel[i+1]) slope[i] =  1;
           if (vel[i]<vel[i+1]) slope[i] = -1;
           if (vel[i]>0)
              if (slope[i]==1)
                    velHuu[i] = vel[i];
              else  velHud[i] = vel[i];
           if (vel[i]<0)
              if (slope[i]==1)
                    velHdu[i] = vel[i];
              else  velHdd[i] = vel[i];
     }
   return(0);
   }      
      
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (i=limit;i>=0; i--)
   {
       int y = iBarShift(NULL,timeFrame,Time[i]);
          vel[i]    = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",ShortLimit,LongLimit,Hot,Price,Original,CfbDepth,CfbSmooth,CfbNormLength,4,y);
          slope[i]  = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",ShortLimit,LongLimit,Hot,Price,Original,CfbDepth,CfbSmooth,CfbNormLength,6,y);
          trend[i]  = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",ShortLimit,LongLimit,Hot,Price,Original,CfbDepth,CfbSmooth,CfbNormLength,7,y);
          velHuu[i] = EMPTY_VALUE;
          velHud[i] = EMPTY_VALUE;
          velHdd[i] = EMPTY_VALUE;
          velHdu[i] = EMPTY_VALUE;
          if (vel[i]>0)
             if (slope[i]==1)
                   velHuu[i] = vel[i];
             else  velHud[i] = vel[i];
          if (vel[i]<0)
             if (slope[i]==1)
                   velHdu[i] = vel[i];
             else  velHdd[i] = vel[i];
          
          //
          //
          //
          //
          //
      
          if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

          //
          //
          //
          //
          //

          datetime time = iTime(NULL,timeFrame,y);
             for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
             for(int k = 1; k < n; k++)
             {
                vel[i+k] = vel[i] + (vel[i+n] - vel[i]) * k/n;
                    
             }                  
        }
        
        for (i=limit; i>=0; i--)
        {
           if (velHuu[i] != EMPTY_VALUE) velHuu[i] = vel[i];
           if (velHud[i] != EMPTY_VALUE) velHud[i] = vel[i];
           if (velHdu[i] != EMPTY_VALUE) velHdu[i] = vel[i];
           if (velHdd[i] != EMPTY_VALUE) velHdd[i] = vel[i];
        }
        
return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double workT3[][12];
double workT3Coeffs[][6];
#define _period 0
#define _c1     1
#define _c2     2
#define _c3     3
#define _c4     4
#define _alpha  5

//
//
//
//
//

double iT3(double price, double period, double hot, bool original, int i, int instanceNo=0)
{
   if (ArrayRange(workT3,0) != Bars)                ArrayResize(workT3,Bars);
   if (ArrayRange(workT3Coeffs,0) < (instanceNo+1)) ArrayResize(workT3Coeffs,instanceNo+1);

   if (workT3Coeffs[instanceNo][_period] != period)
   {
     workT3Coeffs[instanceNo][_period] = period;
        double a = hot;
            workT3Coeffs[instanceNo][_c1] = -a*a*a;
            workT3Coeffs[instanceNo][_c2] = 3*a*a+3*a*a*a;
            workT3Coeffs[instanceNo][_c3] = -6*a*a-3*a-3*a*a*a;
            workT3Coeffs[instanceNo][_c4] = 1+3*a+a*a*a+3*a*a;
            if (original)
                 workT3Coeffs[instanceNo][_alpha] = 2.0/(1.0 + period);
            else workT3Coeffs[instanceNo][_alpha] = 2.0/(2.0 + (period-1.0)/2.0);
   }
   
   //
   //
   //
   //
   //
   
   int buffer = instanceNo*6;
   int r = Bars-i-1;
   if (r == 0)
      {
         workT3[r][0+buffer] = price;
         workT3[r][1+buffer] = price;
         workT3[r][2+buffer] = price;
         workT3[r][3+buffer] = price;
         workT3[r][4+buffer] = price;
         workT3[r][5+buffer] = price;
      }
   else
      {
         workT3[r][0+buffer] = workT3[r-1][0+buffer]+workT3Coeffs[instanceNo][_alpha]*(price              -workT3[r-1][0+buffer]);
         workT3[r][1+buffer] = workT3[r-1][1+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][0+buffer]-workT3[r-1][1+buffer]);
         workT3[r][2+buffer] = workT3[r-1][2+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][1+buffer]-workT3[r-1][2+buffer]);
         workT3[r][3+buffer] = workT3[r-1][3+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][2+buffer]-workT3[r-1][3+buffer]);
         workT3[r][4+buffer] = workT3[r-1][4+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][3+buffer]-workT3[r-1][4+buffer]);
         workT3[r][5+buffer] = workT3[r-1][5+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][4+buffer]-workT3[r-1][5+buffer]);
      }

   //
   //
   //
   //
   //
   
   return(workT3Coeffs[instanceNo][_c1]*workT3[r][5+buffer] + 
          workT3Coeffs[instanceNo][_c2]*workT3[r][4+buffer] + 
          workT3Coeffs[instanceNo][_c3]*workT3[r][3+buffer] + 
          workT3Coeffs[instanceNo][_c4]*workT3[r][2+buffer]);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int    Depths[] = {2,3,4,6,8,12,16,24,32,48,64,96,128,192};
double workCfb[][28];

//
//
//
//
//

double iCfb(double price, int depth, int smooth, int r)
{
   if (ArrayRange(workCfb,0) != Bars) ArrayResize(workCfb,Bars);
   int i = Bars-r-1;
         
   //
   //
   //
   //
   //

   double suma     = 0;
   double sumb     = 0;
   double cfb      = 0;
   double evenCoef = 1;
   double oddCoef  = 1;
   double avg      = 0;
         
      if (i>=smooth)
         for (int k=depth-1; k>=0; k--)
         {
            workCfb[i][k]    = iCfbFunc(price,i,Depths[k],k);
            workCfb[i][k+14] = workCfb[i-1][k+14] + (workCfb[i][k]-workCfb[i-smooth][k])/smooth;

                  if ((k%2)==0)
                        { avg = oddCoef  * workCfb[i][k+14]; oddCoef  = oddCoef  * (1 - avg); }
                  else  { avg = evenCoef * workCfb[i][k+14]; evenCoef = evenCoef * (1 - avg); }
               
               suma += avg*avg*Depths[k];
               sumb += avg;
         }
      else for (k=depth-1; k>=0; k--) { workCfb[i][k] = 0; workCfb[i][k+14] = 0; }            

   //
   //
   //
   //
   //

   if (sumb != 0) cfb = suma/sumb;
   return(cfb);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double  workCfbFunc[][70];
#define _prices 0
#define _roc    1
#define _value1 2
#define _value2 3
#define _value3 4

//
//
//
//

double iCfbFunc(double price, int r, int depth, int k)
{
   k *= 5;
      if (ArrayRange(workCfbFunc,0) != Bars) ArrayResize(workCfbFunc,Bars);
      if (r<=(depth+1))
      {
         workCfbFunc[r][k+_prices] = 0;
         workCfbFunc[r][k+_roc]    = 0;
         workCfbFunc[r][k+_value1] = 0;
         workCfbFunc[r][k+_value2] = 0;
         workCfbFunc[r][k+_value3] = 0;
         return(0);
      }         
      workCfbFunc[r][k+_prices] = price; 

   //
   //
   //
   //
   //

      workCfbFunc[r][k+_roc]    = MathAbs(workCfbFunc[r][k+_prices] - workCfbFunc[r-1][k+_prices]);
      workCfbFunc[r][k+_value1] = workCfbFunc[r-1][k+_value1] - workCfbFunc[r-depth][k+_roc] + workCfbFunc[r][k+_roc];
      workCfbFunc[r][k+_value2] = workCfbFunc[r-1][k+_value2] - workCfbFunc[r-1][k+_value1] + workCfbFunc[r][k+_roc]*depth;
      workCfbFunc[r][k+_value3] = workCfbFunc[r-1][k+_value3] - workCfbFunc[r-1-depth][k+_prices] + workCfbFunc[r-1][k+_prices];
   
      double dividend = MathAbs(depth*workCfbFunc[r][k+_prices]-workCfbFunc[r][k+_value3]);

      //
      //
      //
      //
      //
         
   if (workCfbFunc[r][k+_value2] != 0)
         return( dividend / workCfbFunc[r][k+_value2]);
   else  return(0.00);            
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
return(s);
}

      